﻿#include <iostream>
#include <sstream>
#include <cstdlib>
#include <ctime>
#include "エラー定数.h"
#include "推定.h"
#include "判定.h"
#include "順列.h"
#include "推定.h"

/*! \file main.cpp
    \brief main関数や基本処理の一部が含まれています。

ほかにも概要の定義がここに記されています。
*/

/*! \mainpage 解説

勝手にヒットアンドブローゲームを行って勝手に終了します。
回答側は完全総当たりよりはマシ程度の最低限の動作しか行っていません。
Doxygen 日本語化のサンプルの為に突貫工事で作ったものですので、ええ。

当然ながら日本語の関数名やクラス名が通る環境でしかビルドできません。
また、一部に C++11 の構文があるため、そちらにも対応したものが必要です。自動型推論の auto を使っているだけですが。

Visual Studio 2010 は上記の条件を満たしており、これをビルドできます。
Clang 3.3 以降であればやはりビルドできる可能性があります。ただし BOM
付きの UTF-8 を使っているのでそっちでひっかかるかもしれません。

GCC(g++)は非ASCII文字の関数名などが通らないのでビルドできないはずです。

見どころないし注意点は次の通りです。
\li 継承関係にないクラス同士であっても、同名の読みは一方を指定すれば省略できる
\li \a 名前.cpp に対して \a \\sk \a なまえ.cpp を指定しておけばクラス名その他に 名前 と定義されたものは読みを省略できる
\li 言語要素として以外の複合語はすべて全体に読みを付け直す必要がある
*/

/*! \brief 指定引数に基づき乱数で解答を一つ生成します
    \sk かいとうのさくせい

\param 元数 各桁が取りうる値の種類数です、各桁には 0 - (元数-1) のいずれかが入ります
\param 桁数 返される文字数はこの値と一致します
\return 桁同士が重複しない値からなる \a 桁数 の指定値と同じ長さの配列
*/
std::vector<unsigned int> 解答の作成( const unsigned int 元数 = 9, const unsigned int 桁数 = 4 )
{
	if ( 元数 < 桁数 )
	{
		std::wcerr << L"元の数が桁数より少ないため解答を作成できません"
			<< L"(元数=" << 元数 << L", 桁数=" << 桁数 << L")。" << std::endl;
		exit( 桁数異常 );
	}
	std::vector<unsigned int> 未使用;
	for ( unsigned int i = 0; i < 元数; ++i ) { 未使用.push_back( i ); }
	std::vector<unsigned int> r;
	for ( unsigned int i = 0; i < 桁数; ++i )
	{
		const int idx = std::rand() % 未使用.size();
		const auto it = 未使用.begin() + idx;
		r.push_back( *it );
		未使用.erase( it );
	}
	return r;
}


/*! \brief 整数の配列から可視化した文字列を返します
    \sk かいとうもじか

\param s 元となる配列
\param base 0 に相当する文字を指定ください、配列内の要素がそのまま文字符号として加算され文字化されます
\return 可視化した文字列を返します
*/
std::wstring 解答文字化( const std::vector<unsigned int>& s, const wchar_t base = L'1' )
{
	std::wstring r;
	for ( auto it = s.rbegin(); it != s.rend(); ++it )
	{
		r += wchar_t( base + *it );
	}
	return r;
}


/*! \brief 判定結果を n ヒット m ブローの形で表します
    \sk はんていもじか

\param s 判定結果を表す構造体を指定ください
\return 可視化した文字列を返します
*/
std::wstring 判定文字化( const 判定結果& s )
{
	std::wostringstream ss;
	ss << s.mHit << L"ヒット " << s.mBlow << L"ブロー";
	return ss.str();
}


/*! \brief std::wcout をまともに使えるように初期化します
    \sk ひょうじゅんにゅうしゅつりょくにほんごか

Visual Studio 2010 あるいはその他のコンパイラにあっても
std::wcout は ASCII 外の文字があるとそれを受け付けないばかりか
それ以降の出力を無視してしまいます。

この関数は日本語に対応できるように標準入出力を初期化するものです。
*/
void 標準入出力日本語化()
{
	std::ios_base::sync_with_stdio(false);
	std::locale default_loc("");
	std::locale::global(default_loc);
	std::locale ctype_default(std::locale::classic(), default_loc, std::locale::ctype);
	std::wcout.imbue(ctype_default);
	std::wcerr.imbue(ctype_default);
	std::wcin.imbue(ctype_default);
}


/*! \brief ここから処理が始まります

特に何の変哲もない main 関数です。
\return 正常なら 0 、異常終了したときはそれ以外を返します
\sa エラー定数
*/
int main()
{
	標準入出力日本語化();

	std::srand( static_cast<unsigned int>( std::time(0) ) );

	const unsigned int 元数 = 9;
	const unsigned int 取得長さ = 4;
	
	auto 解答 = 解答の作成( 元数, 取得長さ );
	auto 解答文字 = 解答文字化( 解答 );
	std::wcout << L"解答 = ";
	std::wcout << 解答文字 << std::endl;

	推定 推定器( 元数, 取得長さ );

	for ( int i = 0; i < 1000000; ++i )
	{
		auto 予想 = 推定器.次();
		if ( 予想.size() == 0 )
		{
			std::wcout << L"予想が尽きてしまいました。" << std::endl;
			getchar();
			exit( 予想消沈 );
		}
		auto r = 判定( 解答, 予想 );
		if ( 判定結果::m最終エラー != エラーなし )
		{
			getchar();
			exit( 判定結果::m最終エラー );
		}
		std::wcout << i << L"回目: " << 解答文字化( 予想 ) << L"->"
			<< 判定文字化( r ) << L"\r";
		推定器.結果登録( 予想, r );
		if ( r.mHit == 4 )
		{
			break;
		}
	}
	std::wcout << std::endl
		<< L"正解を出せました。Enterキーで終了します。" << std::endl;
	getchar();
	return 0;
}
