﻿#include <algorithm>
#include <iostream>
#include "順列.h"

/*! \file 順列.cpp
    \brief 順列や組み合わせの処理に使われる処理の定義が含まれています

\sa 順列.h
*/


/*! \brief 階乗もしくは階乗の部分積を返します
   \sk かいじょう

unsigned int でしか計算していないので大きすぎる値にならないように
引数にご注意ください。

\param 値 積を構成する最後の値を指定ください
\param 開始値 積を構成する最初の値を指定ください
\return 値!/(開始値-1)! を返します
*/
unsigned int 階乗( const unsigned int 値, const unsigned int 開始値 = 1 )
{
	if ( 値 <= 開始値 ) { return 1; }
	return 値 * 階乗( 値 - 1, 開始値 );
}


/*! \brief 順列や組み合わせの一巡内容を表示します、デバッグ用です
    \sk ぜんひょうじほんたい

出力先は std::wcout になっています。日本語表示ができるように imbue などで
事前に設定を行っておいてください。

\tparam クラス 順列 もしくは 組み合わせ
\param ptr 当該実体へのポインタ
\sa 順列::全表示(), 組み合わせ::全表示()
*/
template<class T> void 全表示本体( T* ptr )
{
	ptr->初期化();
	int cnt = 1;
	for ( std::vector<unsigned int> seq = ptr->次(); seq.size(); seq = ptr->次() )
	{
		std::wcout << cnt++ << L":";
		for ( auto it = seq.rbegin(); it != seq.rend(); ++it )
		{
			std::wcout << wchar_t( L'a' + *it );
		}
		std::wcout << std::endl;
	}
	std::wcout << L"あるべき長さ = " << ptr->総数() << std::endl;
}


/*! \brief 構築子

\param 元数 使う値の数です、0 から 元数-1 までの整数閉区間が要素として用いられます
\param 取得長さ 毎回返される集合の寸法です、0 のときは 元数 と同じと見なされます
*/
順列::順列( const unsigned int 元数, const unsigned int 取得長さ )
	: m元数( 元数 ), m取得長さ( 取得長さ ? 取得長さ : 元数 )
{
}


/*! \brief 次の配列を生成して返します
    \sk つぎ

内部状態を参照し、前回返した配列の次のものを返します。
添え字の小さいほうを下位桁と考えたときの昇順で配列が生成されます。

\return 生成した次の配列、ただし長さが 0 の場合は一巡したことを示します
*/
std::vector<unsigned int> 順列::次()
{
	// 各桁の値が重複しない昇順という感じの単純な実装です。
	// ここでは添え字の小さいほうが下位桁相当。
	if ( m現況.size() == 0 )
	{
		for ( unsigned int i = 0; i < m取得長さ; ++i )
		{
			m現況.push_back( m取得長さ - 1 - i );
		}
		return m現況;
	}
	std::vector<unsigned int> sub( m現況 );
	auto subIt = sub.begin();
	for ( unsigned int 桁 = 0; 桁 < m取得長さ; ++桁 )
	{
		++subIt;
		for ( unsigned int 値 = m現況[ 桁 ] + 1; 値 < m元数; ++値 )
		{
			if ( std::find( subIt, sub.end(), 値 ) != sub.end() )
			{
				continue;
			}
			m現況[ 桁 ] = 値;
			unsigned int c = 0;
			for ( unsigned int k = 桁; k > 0; --k )
			{
				for ( ; ; ++c )
				{
					if ( c != 値 && std::find( subIt, sub.end(), c ) == sub.end() )
					{
						m現況[ k - 1 ] = c++;
						break;
					}
				}
			}
			return m現況;
		}
	}
	m現況.clear();
	return m現況;
}


/*! \brief 構築子で与えられた設定において幾通りの解答があるかを返します
    \sk そうすう

\return 元数!/(元数-取得長さ)! を返します、桁あふれにご注意ください
*/
unsigned int 順列::総数() const
{
	return 階乗( m元数, m元数 - m取得長さ );
}


/*! \brief すべての解答を構築し、標準出力に流します、デバッグ用です
    \sk ぜんひょうじ

std::wcout に出力を流し込みます。事前に imbue 等必要な処理を行っておいてください。
呼び出し後は初期化状態となります。また、呼び出し前の状態は無視されます。
*/
void 順列::全表示()
{
	全表示本体( this );
}


/*! \brief 構築子

\param 元数 使う値の数です、0 から 元数-1 までの整数閉区間が要素として用いられます
\param 取得長さ 毎回返される集合の寸法です、0 のときは 元数 と同じと見なされます
*/
組み合わせ::組み合わせ( const unsigned int 元数, const unsigned int 取得長さ )
	: m順列( 元数, 取得長さ )
{
}


/*! \brief 次の配列を生成して返します

内部状態を参照し、前回返した配列の次のものを返します。
この実装では添え字の小さいほうから昇順になっている配列を常に返します。

\return 生成した次の配列、ただし長さが 0 の場合は一巡したことを示します
*/

std::vector<unsigned int> 組み合わせ::次()
{
	// 順列の中から各桁の値が昇順になっているものだけ返しています。
	std::vector<unsigned int> r( m順列.次() );
	for ( ; r.size(); r = m順列.次() )
	{
		auto it = r.begin();
		unsigned int prev = *it;
		for ( ++it; it != r.end(); ++it )
		{
			if ( prev <= *it ) { break; }
			prev = *it;
		}
		if ( it == r.end() ) { break; } 
	}
	return r;
}


/*! \brief 構築子で与えられた設定において幾通りの解答があるかを返します

\return 元数!/((元数-取得長さ)!取得長さ!) を返します、桁あふれにご注意ください
*/
unsigned int 組み合わせ::総数() const
{
	return 階乗( m順列.m元数, m順列.m元数 - m順列.m取得長さ ) / 階乗( m順列.m取得長さ );
}


/*! \brief すべての解答を構築し、標準出力に流します、デバッグ用です

std::wcout に出力を流し込みます。事前に imbue 等必要な処理を行っておいてください。
呼び出し後は初期化状態となります。また、呼び出し前の状態は無視されます。
*/
void 組み合わせ::全表示()
{
	全表示本体( this );
}
